#!/usr/bin/env python

try:
  import gobject  # Python 2.x
except:
  from gi.repository import GLib as gobject # Python 3.x
import platform
import logging
import sys
import os
import requests # for http GET
try:
  import thread   # for daemon = True  / Python 2.x
except:
  import _thread as thread   # for daemon = True  / Python 3.x

# our own packages
sys.path.insert(1, os.path.join(os.path.dirname(__file__), '../ext/velib_python'))
from vedbus import VeDbusService

path_UpdateIndex = '/UpdateIndex'


class NodeRedMeterEinspeisung:
  def __init__(self, servicename, deviceinstance, paths, productname='Einspeisung', connection='NodeRED local'):
    self._dbusservice = VeDbusService(servicename)
    self._paths = paths

    logging.debug("%s /DeviceInstance = %d" % (servicename, deviceinstance))

    # Create the management objects, as specified in the ccgx dbus-api document
    self._dbusservice.add_path('/Mgmt/ProcessName', __file__)
    self._dbusservice.add_path('/Mgmt/Connection', connection)

    # Create the mandatory objects
    self._dbusservice.add_path('/DeviceInstance', deviceinstance)
    self._dbusservice.add_path('/ProductId', 0xB017) # like a EM24
    self._dbusservice.add_path('/DeviceType', 72) # like a EM24
    self._dbusservice.add_path('/ProductName', productname)
    self._dbusservice.add_path('/FirmwareVersion', 1.0)
    self._dbusservice.add_path('/HardwareVersion', 0)
    self._dbusservice.add_path('/Connected', 1)

    for path, settings in self._paths.items():
      self._dbusservice.add_path(
        path, settings['initial'], writeable=True, onchangecallback=self._handlechangedvalue)

    gobject.timeout_add(1000, self._update) # pause 1000ms before the next request

  def _update(self):
    try:
      self._dbusservice['/ErrorCode'] = 0
      nodered_url = "http://localhost:1880/meters"
      nodered_r = requests.get(url=nodered_url) # request data from the Fronius PV inverter
      nodered_data = nodered_r.json() # convert JSON data
      self._dbusservice['/Ac/Power'] = nodered_data['einspeisung']['total_power']
      self._dbusservice['/Ac/Current'] = nodered_data['einspeisung']['total_current']
      self._dbusservice['/Ac/Energy/Forward'] = nodered_data['einspeisung']['total_import']
      self._dbusservice['/Ac/Energy/Reverse'] = nodered_data['einspeisung']['total_export']
      self._dbusservice['/Ac/L1/Voltage'] = nodered_data['einspeisung']['l1_voltage']
      self._dbusservice['/Ac/L2/Voltage'] = nodered_data['einspeisung']['l2_voltage']
      self._dbusservice['/Ac/L3/Voltage'] = nodered_data['einspeisung']['l3_voltage']
      self._dbusservice['/Ac/L1/Current'] = nodered_data['einspeisung']['l1_current']
      self._dbusservice['/Ac/L2/Current'] = nodered_data['einspeisung']['l2_current']
      self._dbusservice['/Ac/L3/Current'] = nodered_data['einspeisung']['l3_current']
      self._dbusservice['/Ac/L1/Power'] = nodered_data['einspeisung']['l1_power']
      self._dbusservice['/Ac/L2/Power'] = nodered_data['einspeisung']['l2_power']
      self._dbusservice['/Ac/L3/Power'] = nodered_data['einspeisung']['l3_power']
      self._dbusservice['/Ac/L1/Energy/Forward'] = nodered_data['einspeisung']['l1_import']
      self._dbusservice['/Ac/L2/Energy/Forward'] = nodered_data['einspeisung']['l2_import']
      self._dbusservice['/Ac/L3/Energy/Forward'] = nodered_data['einspeisung']['l3_import']
      self._dbusservice['/Ac/L1/Energy/Reverse'] = nodered_data['einspeisung']['l1_export']
      self._dbusservice['/Ac/L2/Energy/Reverse'] = nodered_data['einspeisung']['l2_export']
      self._dbusservice['/Ac/L3/Energy/Reverse'] = nodered_data['einspeisung']['l3_export']
      
      logging.debug("House Consumption (/Ac/Power): %s" % (self._dbusservice['/Ac/Power']))
      logging.debug("House Forward (/Ac/Energy/Forward): %s" % (self._dbusservice['/Ac/Energy/Forward']))
      logging.debug("House Reverse (/Ac/Energy/Revers): %s" % (self._dbusservice['/Ac/Energy/Reverse']))
      logging.debug("---");

      index = self._dbusservice[path_UpdateIndex] + 1  # increment index
      if index > 255:   # maximum value of the index
        index = 0       # overflow from 255 to 0
      self._dbusservice[path_UpdateIndex] = index
    
    except Exception as e:
      logging.critical("WARNING: Could not read from Node Red, check if Node Red service is running")
      logging.critical('Error at %s', '_update', exc_info=e)
      self._dbusservice['/ErrorCode'] = 1
      self._dbusservice['/Ac/Power'] = 0
      self._dbusservice['/Ac/Current'] = 0
      self._dbusservice['/Ac/L1/Voltage'] = 0
      self._dbusservice['/Ac/L2/Voltage'] = 0
      self._dbusservice['/Ac/L3/Voltage'] = 0
      self._dbusservice['/Ac/L1/Current'] = 0
      self._dbusservice['/Ac/L2/Current'] = 0
      self._dbusservice['/Ac/L3/Current'] = 0
      self._dbusservice['/Ac/L1/Power'] = 0
      self._dbusservice['/Ac/L2/Power'] = 0
      self._dbusservice['/Ac/L3/Power'] = 0
    
    return True

  def _handlechangedvalue(self, path, value):
    logging.debug("someone else updated %s to %s" % (path, value))
    return True # accept the change

def main():
  logging.basicConfig(      format='%(asctime)s,%(msecs)d %(name)s %(levelname)s %(message)s',
                            datefmt='%Y-%m-%d %H:%M:%S',
                            level=logging.ERROR,
                            handlers=[
                                logging.FileHandler("%s/current.log" % (os.path.dirname(os.path.realpath(__file__)))),
                                logging.StreamHandler()
                            ])
  thread.daemon = True # allow the program to quit
  try:
      from dbus.mainloop.glib import DBusGMainLoop
      # Have a mainloop, so we can send/receive asynchronous calls to and from dbus
      DBusGMainLoop(set_as_default=True)
      _kwh = lambda p, v: (str(round(v, 2)) + ' KWh')
      _A = lambda p, v: (str(round(v, 1)) + ' A')
      _W = lambda p, v: (str(round(v, 1)) + ' W')
      _V = lambda p, v: (str(round(v, 1)) + ' V')  


      pvac_output = NodeRedMeterEinspeisung(
          servicename='com.victronenergy.grid',
          deviceinstance=40,
          paths={
          '/ErrorCode': {'initial': 0},
          '/Ac/Power': {'initial': 0, 'textformat': _W},
          '/Ac/Current': {'initial': 0, 'textformat': _A},
          '/Ac/L1/Voltage': {'initial': 0, 'textformat': _V},
          '/Ac/L2/Voltage': {'initial': 0, 'textformat': _V},
          '/Ac/L3/Voltage': {'initial': 0, 'textformat': _V},
          '/Ac/L1/Current': {'initial': 0, 'textformat': _A},
          '/Ac/L2/Current': {'initial': 0, 'textformat': _A},
          '/Ac/L3/Current': {'initial': 0, 'textformat': _A},
          '/Ac/L1/Power': {'initial': 0, 'textformat': _W},
          '/Ac/L2/Power': {'initial': 0, 'textformat': _W},
          '/Ac/L3/Power': {'initial': 0, 'textformat': _W},
          '/Ac/Energy/Forward': {'initial': 0, 'textformat': _kwh},
          '/Ac/Energy/Reverse': {'initial': 0, 'textformat': _kwh},
          '/Ac/L1/Energy/Forward': {'initial': 0, 'textformat': _kwh},
          '/Ac/L2/Energy/Forward': {'initial': 0, 'textformat': _kwh},
          '/Ac/L3/Energy/Forward': {'initial': 0, 'textformat': _kwh},
          '/Ac/L1/Energy/Reverse': {'initial': 0, 'textformat': _kwh},
          '/Ac/L2/Energy/Reverse': {'initial': 0, 'textformat': _kwh},
          '/Ac/L3/Energy/Reverse': {'initial': 0, 'textformat': _kwh},
      
          path_UpdateIndex: {'initial': 0},
          })

      logging.info('Connected to dbus, and switching over to gobject.MainLoop() (= event based)')
      mainloop = gobject.MainLoop()
      mainloop.run()
  
  except Exception as e:
    logging.critical('Error at %s', 'main', exc_info=e)

if __name__ == "__main__":
  main()